@@ -10,13 +10,10 @@ import (
 	"sync"
 	"time"
 
-	"github.com/blang/semver"
 	"github.com/google/uuid"
-	"github.com/pkg/errors"
 	corev1 "k8s.io/api/core/v1"
 	apierrors "k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
-	"k8s.io/apimachinery/pkg/apis/meta/v1/unstructured"
 	utilruntime "k8s.io/apimachinery/pkg/util/runtime"
 	"k8s.io/apimachinery/pkg/util/wait"
 	informerscorev1 "k8s.io/client-go/informers/core/v1"
@@ -29,7 +26,7 @@ import (
 	"k8s.io/client-go/tools/record"
 	"k8s.io/client-go/transport"
 	"k8s.io/client-go/util/workqueue"
-	"k8s.io/klog"
+	"k8s.io/klog/v2"
 
 	configv1 "github.com/openshift/api/config/v1"
 	clientset "github.com/openshift/client-go/config/clientset/versioned"
@@ -49,14 +46,11 @@ import (
 	"github.com/openshift/cluster-version-operator/pkg/verify/store"
 	"github.com/openshift/cluster-version-operator/pkg/verify/store/configmap"
 	"github.com/openshift/cluster-version-operator/pkg/verify/store/serial"
+	"github.com/openshift/cluster-version-operator/pkg/verify/store/sigstore"
 )
 
 const (
-	// maxRetries is the number of times a machineconfig pool will be retried before it is dropped out of the queue.
-	// With the current rate-limiter in use (5ms*2^(maxRetries-1)) the following numbers represent the times
-	// a machineconfig pool is going to be requeued:
-	//
-	// 5ms, 10ms, 20ms, 40ms, 80ms, 160ms, 320ms, 640ms, 1.3s, 2.6s, 5.1s, 10.2s, 20.4s, 41s, 82s
+	// maxRetries is the number of times a work-item will be retried before it is dropped out of the queue.
 	maxRetries = 15
 )
 
@@ -86,13 +80,13 @@ type Operator struct {
 	// namespace and name are used to find the ClusterVersion, OperatorStatus.
 	namespace, name string
 
-	// releaseImage is the image the current operator points to and allows
-	// templating of the CVO deployment manifest.
-	releaseImage string
-	// releaseVersion is a string identifier for the current version, read
-	// from the image of the operator. It may be empty if no version exists, in
-	// which case no available updates will be returned.
-	releaseVersion string
+	// release is the release the current operator points to and
+	// metadata read from the release image.  It allows templating of
+	// the CVO deployment manifest.
+	//
+	// Fetch via currentVersion() to populate metadata from
+	// availableUpdates.
+	release configv1.Release
 	// releaseCreated, if set, is the timestamp of the current update.
 	releaseCreated time.Time
 
@@ -114,11 +108,12 @@ type Operator struct {
 	// syncBackoff allows the tests to use a quicker backoff
 	syncBackoff wait.Backoff
 
-	cvLister       configlistersv1.ClusterVersionLister
-	coLister       configlistersv1.ClusterOperatorLister
-	cmConfigLister listerscorev1.ConfigMapNamespaceLister
-	proxyLister    configlistersv1.ProxyLister
-	cacheSynced    []cache.InformerSynced
+	cvLister              configlistersv1.ClusterVersionLister
+	coLister              configlistersv1.ClusterOperatorLister
+	cmConfigLister        listerscorev1.ConfigMapNamespaceLister
+	cmConfigManagedLister listerscorev1.ConfigMapNamespaceLister
+	proxyLister           configlistersv1.ProxyLister
+	cacheSynced           []cache.InformerSynced
 
 	// queue tracks applying updates to a cluster.
 	queue workqueue.RateLimitingInterface
@@ -168,6 +163,7 @@ func New(
 	cvInformer configinformersv1.ClusterVersionInformer,
 	coInformer configinformersv1.ClusterOperatorInformer,
 	cmConfigInformer informerscorev1.ConfigMapInformer,
+	cmConfigManagedInformer informerscorev1.ConfigMapInformer,
 	proxyInformer configinformersv1.ProxyInformer,
 	client clientset.Interface,
 	kubeClient kubernetes.Interface,
@@ -178,10 +174,12 @@ func New(
 	eventBroadcaster.StartRecordingToSink(&coreclientsetv1.EventSinkImpl{Interface: kubeClient.CoreV1().Events(namespace)})
 
 	optr := &Operator{
-		nodename:     nodename,
-		namespace:    namespace,
-		name:         name,
-		releaseImage: releaseImage,
+		nodename:  nodename,
+		namespace: namespace,
+		name:      name,
+		release: configv1.Release{
+			Image: releaseImage,
+		},
 
 		enableDefaultClusterVersion: enableDefaultClusterVersion,
 
@@ -211,6 +209,7 @@ func New(
 
 	optr.proxyLister = proxyInformer.Lister()
 	optr.cmConfigLister = cmConfigInformer.Lister().ConfigMaps(internal.ConfigNamespace)
+	optr.cmConfigManagedLister = cmConfigManagedInformer.Lister().ConfigMaps(internal.ConfigManagedNamespace)
 
 	// make sure this is initialized after all the listers are initialized
 	optr.upgradeableChecks = optr.defaultUpgradeableChecks()
@@ -218,41 +217,26 @@ func New(
 	return optr
 }
 
-// verifyClientBuilder is a wrapper around the operator's HTTPClient method.
-// It is used by the releaseVerifier to get an up-to-date http client.
-type verifyClientBuilder struct {
-	builder func() (*http.Client, error)
-}
-
-func (vcb *verifyClientBuilder) HTTPClient() (*http.Client, error) {
-	return vcb.builder()
-}
-
 // InitializeFromPayload retrieves the payload contents and verifies the initial state, then configures the
 // controller that loads and applies content to the cluster. It returns an error if the payload appears to
 // be in error rather than continuing.
 func (optr *Operator) InitializeFromPayload(restConfig *rest.Config, burstRestConfig *rest.Config) error {
-	update, err := payload.LoadUpdate(optr.defaultPayloadDir(), optr.releaseImage, optr.exclude)
+	update, err := payload.LoadUpdate(optr.defaultPayloadDir(), optr.release.Image, optr.exclude)
 	if err != nil {
 		return fmt.Errorf("the local release contents are invalid - no current version can be determined from disk: %v", err)
 	}
-	// XXX: set this to the cincinnati version in preference
-	if _, err := semver.Parse(update.ImageRef.Name); err != nil {
-		return fmt.Errorf("the local release contents name %q is not a valid semantic version - no current version will be reported: %v", update.ImageRef.Name, err)
-	}
 
+	optr.release = update.Release
 	optr.releaseCreated = update.ImageRef.CreationTimestamp.Time
-	optr.releaseVersion = update.ImageRef.Name
 
-	// Wraps operator's HTTPClient method to allow releaseVerifier to create http client with up-to-date config.
-	clientBuilder := &verifyClientBuilder{builder: optr.HTTPClient}
+	httpClientConstructor := sigstore.NewCachedHTTPClientConstructor(optr.HTTPClient, nil)
 	configClient, err := coreclientsetv1.NewForConfig(restConfig)
 	if err != nil {
 		return fmt.Errorf("unable to create a configuration client: %v", err)
 	}
 
 	// attempt to load a verifier as defined in the payload
-	verifier, signatureStore, err := loadConfigMapVerifierDataFromUpdate(update, clientBuilder, configClient)
+	verifier, signatureStore, err := loadConfigMapVerifierDataFromUpdate(update, httpClientConstructor.HTTPClient, configClient)
 	if err != nil {
 		return err
 	}
@@ -278,6 +262,7 @@ func (optr *Operator) InitializeFromPayload(restConfig *rest.Config, burstRestCo
 			Steps:    3,
 		},
 		optr.exclude,
+		optr.eventRecorder,
 	)
 
 	return nil
@@ -287,72 +272,55 @@ func (optr *Operator) InitializeFromPayload(restConfig *rest.Config, burstRestCo
 // It returns an error if the data is not valid, or no verifier if no config map is found. See the verify
 // package for more details on the algorithm for verification. If the annotation is set, a verifier or error
 // is always returned.
-func loadConfigMapVerifierDataFromUpdate(update *payload.Update, clientBuilder verify.ClientBuilder, configMapClient coreclientsetv1.ConfigMapsGetter) (verify.Interface, *verify.StorePersister, error) {
-	configMapGVK := corev1.SchemeGroupVersion.WithKind("ConfigMap")
-	for _, manifest := range update.Manifests {
-		if manifest.GVK != configMapGVK {
-			continue
-		}
-		if _, ok := manifest.Obj.GetAnnotations()[verify.ReleaseAnnotationConfigMapVerifier]; !ok {
-			continue
-		}
-		src := fmt.Sprintf("the config map %s/%s", manifest.Obj.GetNamespace(), manifest.Obj.GetName())
-		data, _, err := unstructured.NestedStringMap(manifest.Obj.Object, "data")
-		if err != nil {
-			return nil, nil, errors.Wrapf(err, "%s is not valid: %v", src, err)
-		}
-		verifier, err := verify.NewFromConfigMapData(src, data, clientBuilder)
-		if err != nil {
-			return nil, nil, err
-		}
-
-		// allow the verifier to consult the cluster for signature data, and also configure
-		// a process that writes signatures back to that store
-		signatureStore := configmap.NewStore(configMapClient, nil)
-		verifier.Store = &serial.Store{Stores: []store.Store{signatureStore, verifier.Store}}
-		persister := verify.NewSignatureStorePersister(signatureStore, verifier)
-		return verifier, persister, nil
+func loadConfigMapVerifierDataFromUpdate(update *payload.Update, clientBuilder sigstore.HTTPClient, configMapClient coreclientsetv1.ConfigMapsGetter) (verify.Interface, *verify.StorePersister, error) {
+	verifier, err := verify.NewFromManifests(update.Manifests, clientBuilder)
+	if err != nil {
+		return nil, nil, err
 	}
-	return nil, nil, nil
+	if verifier == nil {
+		return nil, nil, nil
+	}
+
+	// allow the verifier to consult the cluster for signature data, and also configure
+	// a process that writes signatures back to that store
+	signatureStore := configmap.NewStore(configMapClient, nil)
+	verifier.Store = &serial.Store{Stores: []store.Store{signatureStore, verifier.Store}}
+	persister := verify.NewSignatureStorePersister(signatureStore, verifier)
+	return verifier, persister, nil
 }
 
-// Run runs the cluster version operator until runContext.Done() and
-// then attempts a clean shutdown limited by shutdownContext.Done().
-// Assumes runContext.Done() occurs before or simultaneously with
-// shutdownContext.Done().
-func (optr *Operator) Run(runContext context.Context, shutdownContext context.Context, workers int) error {
+// Run runs the cluster version operator until stopCh is completed. Workers is ignored for now.
+func (optr *Operator) Run(ctx context.Context, workers int) error {
 	defer optr.queue.ShutDown()
-	stopCh := runContext.Done()
+	stopCh := ctx.Done()
 	workerStopCh := make(chan struct{})
 
 	klog.Infof("Starting ClusterVersionOperator with minimum reconcile period %s", optr.minimumUpdateCheckInterval)
 	defer klog.Info("Shutting down ClusterVersionOperator")
 
 	if !cache.WaitForCacheSync(stopCh, optr.cacheSynced...) {
-		return fmt.Errorf("caches never synchronized: %w", runContext.Err())
+		return fmt.Errorf("caches never synchronized: %w", ctx.Err())
 	}
 
 	// trigger the first cluster version reconcile always
 	optr.queue.Add(optr.queueKey())
 
 	// start the config sync loop, and have it notify the queue when new status is detected
-	go runThrottledStatusNotifier(stopCh, optr.statusInterval, 2, optr.configSync.StatusCh(), func() { optr.queue.Add(optr.queueKey()) })
-	go optr.configSync.Start(runContext, 16)
-	go wait.UntilWithContext(runContext, func(runContext context.Context) {
-		optr.worker(runContext, optr.availableUpdatesQueue, optr.availableUpdatesSync)
-	}, time.Second)
-	go wait.UntilWithContext(runContext, func(runContext context.Context) { optr.worker(runContext, optr.upgradeableQueue, optr.upgradeableSync) }, time.Second)
-	go wait.UntilWithContext(runContext, func(runContext context.Context) {
+	go runThrottledStatusNotifier(ctx, optr.statusInterval, 2, optr.configSync.StatusCh(), func() { optr.queue.Add(optr.queueKey()) })
+	go optr.configSync.Start(ctx, 16, optr.name, optr.cvLister)
+	go wait.UntilWithContext(ctx, func(ctx context.Context) { optr.worker(ctx, optr.availableUpdatesQueue, optr.availableUpdatesSync) }, time.Second)
+	go wait.UntilWithContext(ctx, func(ctx context.Context) { optr.worker(ctx, optr.upgradeableQueue, optr.upgradeableSync) }, time.Second)
+	go wait.UntilWithContext(ctx, func(ctx context.Context) {
 		defer close(workerStopCh)
 
 		// run the worker, then when the queue is closed sync one final time to flush any pending status
-		optr.worker(runContext, optr.queue, func(key string) error { return optr.sync(runContext, key) })
-		if err := optr.sync(shutdownContext, optr.queueKey()); err != nil {
+		optr.worker(ctx, optr.queue, func(ctx context.Context, key string) error { return optr.sync(ctx, key) })
+		if err := optr.sync(ctx, optr.queueKey()); err != nil {
 			utilruntime.HandleError(fmt.Errorf("unable to perform final sync: %v", err))
 		}
 	}, time.Second)
 	if optr.signatureStore != nil {
-		go optr.signatureStore.Run(runContext, optr.minimumUpdateCheckInterval*2)
+		go optr.signatureStore.Run(ctx, optr.minimumUpdateCheckInterval*2)
 	}
 
 	<-stopCh
@@ -389,21 +357,21 @@ func (optr *Operator) eventHandler() cache.ResourceEventHandler {
 	}
 }
 
-func (optr *Operator) worker(ctx context.Context, queue workqueue.RateLimitingInterface, syncHandler func(string) error) {
+func (optr *Operator) worker(ctx context.Context, queue workqueue.RateLimitingInterface, syncHandler func(context.Context, string) error) {
 	for processNextWorkItem(ctx, queue, syncHandler, optr.syncFailingStatus) {
 	}
 }
 
 type syncFailingStatusFunc func(ctx context.Context, config *configv1.ClusterVersion, err error) error
 
-func processNextWorkItem(ctx context.Context, queue workqueue.RateLimitingInterface, syncHandler func(string) error, syncFailingStatus syncFailingStatusFunc) bool {
+func processNextWorkItem(ctx context.Context, queue workqueue.RateLimitingInterface, syncHandler func(context.Context, string) error, syncFailingStatus syncFailingStatusFunc) bool {
 	key, quit := queue.Get()
 	if quit {
 		return false
 	}
 	defer queue.Done(key)
 
-	err := syncHandler(key.(string))
+	err := syncHandler(ctx, key.(string))
 	handleErr(ctx, queue, err, key, syncFailingStatus)
 	return true
 }
@@ -415,14 +383,14 @@ func handleErr(ctx context.Context, queue workqueue.RateLimitingInterface, err e
 	}
 
 	if queue.NumRequeues(key) < maxRetries {
-		klog.V(2).Infof("Error syncing operator %v: %v", key, err)
+		klog.V(2).Infof("Error handling %v: %v", key, err)
 		queue.AddRateLimited(key)
 		return
 	}
 
 	err = syncFailingStatus(ctx, nil, err)
 	utilruntime.HandleError(err)
-	klog.V(2).Infof("Dropping operator %q out of the queue %v: %v", key, queue, err)
+	klog.V(2).Infof("Dropping %q out of the queue %v: %v", key, queue, err)
 	queue.Forget(key)
 }
 
@@ -466,7 +434,11 @@ func (optr *Operator) sync(ctx context.Context, key string) error {
 	if ok {
 		klog.V(4).Infof("Desired version from spec is %#v", desired)
 	} else {
-		desired = optr.currentVersion()
+		currentVersion := optr.currentVersion()
+		desired = configv1.Update{
+			Version: currentVersion.Version,
+			Image:   currentVersion.Image,
+		}
 		klog.V(4).Infof("Desired version from operator is %#v", desired)
 	}
 
@@ -500,7 +472,7 @@ func (optr *Operator) sync(ctx context.Context, key string) error {
 
 // availableUpdatesSync is triggered on cluster version change (and periodic requeues) to
 // sync available updates. It only modifies cluster version.
-func (optr *Operator) availableUpdatesSync(key string) error {
+func (optr *Operator) availableUpdatesSync(ctx context.Context, key string) error {
 	startTime := time.Now()
 	klog.V(4).Infof("Started syncing available updates %q (%v)", key, startTime)
 	defer func() {
@@ -517,13 +489,12 @@ func (optr *Operator) availableUpdatesSync(key string) error {
 	if errs := validation.ValidateClusterVersion(config); len(errs) > 0 {
 		return nil
 	}
-
-	return optr.syncAvailableUpdates(config)
+	return optr.syncAvailableUpdates(ctx, config)
 }
 
 // upgradeableSync is triggered on cluster version change (and periodic requeues) to
 // sync upgradeableCondition. It only modifies cluster version.
-func (optr *Operator) upgradeableSync(key string) error {
+func (optr *Operator) upgradeableSync(ctx context.Context, key string) error {
 	startTime := time.Now()
 	klog.V(4).Infof("Started syncing upgradeable %q (%v)", key, startTime)
 	defer func() {
@@ -616,7 +587,7 @@ func (optr *Operator) getOrCreateClusterVersion(ctx context.Context, enableDefau
 }
 
 // versionString returns a string describing the desired version.
-func versionString(update configv1.Update) string {
+func versionString(update configv1.Release) string {
 	if len(update.Version) > 0 {
 		return update.Version
 	}
@@ -626,12 +597,50 @@ func versionString(update configv1.Update) string {
 	return "<unknown>"
 }
 
-// currentVersion returns an update object describing the current known cluster version.
-func (optr *Operator) currentVersion() configv1.Update {
-	return configv1.Update{
-		Version: optr.releaseVersion,
-		Image:   optr.releaseImage,
+// currentVersion returns an update object describing the current
+// known cluster version.  Values from the upstream Cincinnati service
+// are used as fallbacks for any properties not defined in the release
+// image itself.
+func (optr *Operator) currentVersion() configv1.Release {
+	return optr.mergeReleaseMetadata(optr.release)
+}
+
+// mergeReleaseMetadata returns a deep copy of the input release.
+// Values from any matching availableUpdates release are used as
+// fallbacks for any properties not defined in the input release.
+func (optr *Operator) mergeReleaseMetadata(release configv1.Release) configv1.Release {
+	merged := *release.DeepCopy()
+
+	if merged.Version == "" || len(merged.URL) == 0 || merged.Channels == nil {
+		// only fill in missing values from availableUpdates, to avoid clobbering data from payload.LoadUpdate.
+		availableUpdates := optr.getAvailableUpdates()
+		if availableUpdates != nil {
+			var update *configv1.Release
+			if merged.Image == availableUpdates.Current.Image {
+				update = &availableUpdates.Current
+			} else {
+				for _, u := range availableUpdates.Updates {
+					if u.Image == merged.Image {
+						update = &u
+						break
+					}
+				}
+			}
+			if update != nil {
+				if merged.Version == "" {
+					merged.Version = update.Version
+				}
+				if len(merged.URL) == 0 {
+					merged.URL = update.URL
+				}
+				if merged.Channels == nil {
+					merged.Channels = append(update.Channels[:0:0], update.Channels...) // copy
+				}
+			}
+		}
 	}
+
+	return merged
 }
 
 // SetSyncWorkerForTesting updates the sync worker for whitebox testing.
@@ -766,17 +775,15 @@ func (optr *Operator) HTTPClient() (*http.Client, error) {
 // getTransportOpts retrieves the URL of the cluster proxy and the CA
 // trust, if they exist.
 func (optr *Operator) getTransportOpts() (*url.URL, *tls.Config, error) {
-	proxyURL, cmNameRef, err := optr.getHTTPSProxyURL()
+	proxyURL, err := optr.getHTTPSProxyURL()
 	if err != nil {
 		return nil, nil, err
 	}
 
 	var tlsConfig *tls.Config
-	if cmNameRef != "" {
-		tlsConfig, err = optr.getTLSConfig(cmNameRef)
-		if err != nil {
-			return nil, nil, err
-		}
+	tlsConfig, err = optr.getTLSConfig()
+	if err != nil {
+		return nil, nil, err
 	}
 	return proxyURL, tlsConfig, nil
 }
